==> OpenCL-0.8.2.tgz: OpenCL-0.8.2/src/big_code.cpp <==
/*************************************************
* BigInt Encoding/Decoding Source File           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <cctype>
#include <opencl/bigint.h>
#include <opencl/encoder.h>
#include <opencl/numthry.h>

namespace OpenCL {

u32bit encoded_size(const BigInt& n, Encoding code)
   {
   static const double LOG_2_BASE_10 = 0.30102999566;
   if(code == Binary)
      return ((n.bits() + 7) / 8);
   else if(code == Hexadecimal)
      return ((n.bits() + 3) / 4);
   else if(code == Decimal)
      return (u32bit)((n.bits() * LOG_2_BASE_10) + 1);
   else throw Invalid_Argument("Cannot encode BigInt with unknown base");
   }

void encode(byte output[], const BigInt& n, Encoding code)
   {
   if(code == Binary)
      n.binary_encode(output);
   else if(code == Hexadecimal)
      {
      SecureVector<byte> binary = n.binary_encode();
      for(u32bit j = 0; j != binary.size(); j++)
         Hex::encode(binary + j, output + 2*j);
      }
   else if(code == Decimal)
      {
      static const BigInt TEN = 10;
      BigInt copy = n;
      BigInt remainder;
      copy.set_sign(Positive);
      u32bit output_size = encoded_size(n, Decimal);
      for(u32bit j = 0; j != output_size; j++)
         {
         divide(copy, TEN, copy, remainder);
         output[output_size - 1 - j] = remainder.at(0) + '0';
         if(copy.is_zero())
            break;
         }
      }
   else
      throw Invalid_Argument("Cannot encode BigInt with unknown base");
   }

SecureVector<byte> encode(const BigInt& n, Encoding code)
   {
   SecureVector<byte> output(encoded_size(n, code));
   encode(output, n, code);
   if(code != Binary)
      for(u32bit j = 0; j != output.size(); j++)
         if(output[j] == 0)
            output[j] = '0';
   return output;
   }

BigInt decode(const SecureVector<byte>& buf, Encoding code)
   {
   return decode(buf, buf.size(), code);
   }

BigInt decode(const byte buf[], u32bit length, Encoding code)
   {
   BigInt r;
   if(code == Binary)
      r.binary_decode(buf, length);
   else if(code == Hexadecimal)
      {
      if(length % 2)
         throw Format_Error("BigInt hex decode: input must be in full bytes");
      SecureVector<byte> binary(length / 2);
      for(u32bit j = 0; j != binary.size(); j++)
         {
         if(Hex::is_valid(buf[2*j]) && Hex::is_valid(buf[2*j+1]))
            Hex::decode(buf + 2*j, binary[j]);
         else
            throw Invalid_Argument("BigInt: Invalid hexadecimal string");
         }
      r.binary_decode(binary);
      }
   else if(code == Decimal)
      {
      static const BigInt TEN = 10;
      for(u32bit j = 0; j != length; j++)
         {
         byte x = buf[j] - '0';
         if(x > 9)
            throw Invalid_Argument("BigInt: Invalid decimal string");
         r = TEN * r + x;
         }
      }
   else
      throw Invalid_Argument("Cannot decode BigInt with unknown base");
   return r;
   }

}

==> OpenCL-0.8.3.tgz: OpenCL-0.8.3/src/big_code.cpp <==
/*************************************************
* BigInt Encoding/Decoding Source File           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <cctype>
#include <opencl/bigint.h>
#include <opencl/numthry.h>
#include <opencl/hex.h>

namespace OpenCL {

u32bit encoded_size(const BigInt& n, Encoding code)
   {
   static const double LOG_2_BASE_10 = 0.30102999566;
   if(code == Binary)
      return ((n.bits() + 7) / 8);
   else if(code == Hexadecimal)
      return 2 * ((n.bits() + 7) / 8);
   else if(code == Decimal)
      return (u32bit)((n.bits() * LOG_2_BASE_10) + 1);
   else throw Invalid_Argument("Cannot encode BigInt with unknown base");
   }

void encode(byte output[], const BigInt& n, Encoding code)
   {
   if(code == Binary)
      n.binary_encode(output);
   else if(code == Hexadecimal)
      {
      SecureVector<byte> binary = n.binary_encode();
      for(u32bit j = 0; j != binary.size(); j++)
         Hex_Encoder::encode(binary[j], output + 2*j);
      }
   else if(code == Decimal)
      {
      static const BigInt TEN = 10;
      BigInt copy = n;
      BigInt remainder;
      copy.set_sign(Positive);
      u32bit output_size = encoded_size(n, Decimal);
      for(u32bit j = 0; j != output_size; j++)
         {
         divide(copy, TEN, copy, remainder);
         output[output_size - 1 - j] = remainder.at(0) + '0';
         if(copy.is_zero())
            break;
         }
      }
   else
      throw Invalid_Argument("Cannot encode BigInt with unknown base");
   }

SecureVector<byte> encode(const BigInt& n, Encoding code)
   {
   SecureVector<byte> output(encoded_size(n, code));
   encode(output, n, code);
   if(code != Binary)
      for(u32bit j = 0; j != output.size(); j++)
         if(output[j] == 0)
            output[j] = '0';
   return output;
   }

BigInt decode(const SecureVector<byte>& buf, Encoding code)
   {
   return decode(buf, buf.size(), code);
   }

BigInt decode(const byte buf[], u32bit length, Encoding code)
   {
   BigInt r;
   if(code == Binary)
      r.binary_decode(buf, length);
   else if(code == Hexadecimal)
      {
      u32bit offset = (length % 2);
      SecureVector<byte> binary(length / 2 + offset);

      if(offset)
         {
         if(!Hex_Decoder::is_valid(buf[0]))
            throw Invalid_Argument("BigInt: Invalid hexadecimal string");
         byte temp[2] = { '0', buf[0] };
         binary[0] = Hex_Decoder::decode(temp);
         buf++;
         length--;
         }

      for(u32bit j = 0; j != binary.size() - offset; j++)
         {
         if(Hex_Decoder::is_valid(buf[2*j]) &&
            Hex_Decoder::is_valid(buf[2*j+1]))
            binary[j+offset] = Hex_Decoder::decode(buf + 2*j);
         else
            throw Invalid_Argument("BigInt: Invalid hexadecimal string");
         }

      r.binary_decode(binary);
      }
   else if(code == Decimal)
      {
      static const BigInt TEN = 10;
      for(u32bit j = 0; j != length; j++)
         {
         byte x = buf[j] - '0';
         if(x > 9)
            throw Invalid_Argument("BigInt: Invalid decimal string");
         r = TEN * r + x;
         }
      }
   else
      throw Invalid_Argument("Cannot decode BigInt with unknown base");
   return r;
   }

}
